/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.sun.jdi.ArrayReference;
import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.Value;

import java.util.List;

/**
 *
 */
public class XyzzyxJDIArrayReference extends XyzzyxJDIObjectReference implements ArrayReference {
    public XyzzyxJDIArrayReference(XyzzyxJDIVirtualMachine xyzzyxvm, ArrayReference underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public XyzzyxJDIValue getValue(int index) {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().getValue(index));
    }

    @Override
    public List<Value> getValues() {
        return XyzzyxJDIWrapper.wrapValues(virtualMachine(), underlying().getValues());
    }

    @Override
    public List<Value> getValues(int index, int length) {
        return XyzzyxJDIWrapper.wrapValues(virtualMachine(), underlying().getValues(index, length));
    }

    @Override
    public int length() {
        return underlying().length();
    }

    @Override
    public void setValue(int index, Value value)
            throws InvalidTypeException, ClassNotLoadedException {
        underlying().setValue(index, XyzzyxJDIWrapper.unwrap(value));
    }

    @Override
    public void setValues(List<? extends Value> values)
            throws InvalidTypeException, ClassNotLoadedException {
        underlying().setValues(XyzzyxJDIWrapper.unwrapValues(values));
    }

    @Override
    public void setValues(int index, List<? extends Value> values, int srcIndex, int length)
            throws InvalidTypeException, ClassNotLoadedException {
        underlying().setValues(index, XyzzyxJDIWrapper.unwrapValues(values), srcIndex, length);
    }

    @Override
    protected ArrayReference underlying() {
        return (ArrayReference) super.underlying();
    }
}
