/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.ClassType;
import com.sun.jdi.Field;
import com.sun.jdi.IncompatibleThreadStateException;
import com.sun.jdi.InterfaceType;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.InvocationException;
import com.sun.jdi.Method;
import com.sun.jdi.ObjectReference;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.Value;

import java.util.List;
import java.util.ArrayList;

/**
 *
 */
public class XyzzyxJDIClassType extends XyzzyxJDIReferenceType implements ClassType {
    private boolean isXyzzyxTypeSet = false;
    private boolean isXyzzyxType    = false;

    public XyzzyxJDIClassType(XyzzyxJDIVirtualMachine xyzzyxvm, ClassType underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public List<InterfaceType> allInterfaces() {
        return XyzzyxJDIWrapper.wrapInterfaceTypes(virtualMachine(), underlying().allInterfaces());
    }

    @Override
    public XyzzyxJDIMethod concreteMethodByName(String name, String signature) {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().concreteMethodByName(name, signature));
    }

    @Override
    public List<InterfaceType> interfaces() {
        return XyzzyxJDIWrapper.wrapInterfaceTypes(virtualMachine(), underlying().interfaces());
    }

    @Override
    public XyzzyxJDIValue invokeMethod(ThreadReference thread, Method method, List<? extends Value> values, int options)
            throws InvalidTypeException, ClassNotLoadedException, IncompatibleThreadStateException, InvocationException {
        Value value =
                underlying().invokeMethod(
                    XyzzyxJDIWrapper.unwrap(thread), XyzzyxJDIWrapper.unwrap(method),
                    XyzzyxJDIWrapper.unwrapValues(values), options);
        return XyzzyxJDIWrapper.wrap(virtualMachine(), value);
    }

    @Override
    public boolean isEnum() {
        return underlying().isEnum();
    }

    @Override
    public XyzzyxJDIObjectReference newInstance(ThreadReference thread, Method method, List<? extends Value> values, int options)
            throws InvalidTypeException, ClassNotLoadedException, IncompatibleThreadStateException, InvocationException {
         ObjectReference result =
                 underlying().newInstance(
                      XyzzyxJDIWrapper.unwrap(thread), XyzzyxJDIWrapper.unwrap(method),
                      XyzzyxJDIWrapper.unwrapValues(values), options);
         return XyzzyxJDIWrapper.wrap(virtualMachine(), result);
    }


    /**
     * JDI extension:  This will call the setter if one exists.  If an invokeMethod Exception occurs,
     * it is saved in XyzzyxJDIVirtualMachine.
     */
    @Override
    public void setValue(Field field, Value value) throws
        InvalidTypeException, ClassNotLoadedException {

        virtualMachine().setLastFieldAccessException(null);
        Field jdiField = XyzzyxJDIWrapper.unwrap(field);
        Value jdiValue = XyzzyxJDIWrapper.unwrap(value);
        underlying().setValue(jdiField, jdiValue);
    }

    @Override
    public List<ClassType> subclasses() {
        return XyzzyxJDIWrapper.wrapClassTypes(virtualMachine(), underlying().subclasses());
    }

    @Override
    public XyzzyxJDIClassType superclass() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().superclass());
    }

    @Override
    protected ClassType underlying() {
        return (ClassType) super.underlying();
    }
}
