/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.Field;

/**
 *
 */
public class XyzzyxJDIField extends XyzzyxJDITypeComponent implements Field {
    public XyzzyxJDIField(XyzzyxJDIVirtualMachine xyzzyxvm, Field underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public boolean isEnumConstant() {
        return underlying().isEnumConstant();
    }

    @Override
    public boolean isTransient() {
        return underlying().isTransient();
    }

    @Override
    public boolean isVolatile() {
        return underlying().isVolatile();
    }

    @Override
    public XyzzyxJDIType type() throws ClassNotLoadedException {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().type());
    }

    @Override
    public String typeName() {
        return underlying().typeName();
    }

    @Override
    public int compareTo(Field o) {
        return underlying().compareTo(o);
    }

    @Override
    public String toString() {
        // Need the fully qualified name of the class, and the
        // filtered name of the field.
        StringBuilder buf = new StringBuilder();

        buf.append(declaringType().name());
        buf.append('.');
        buf.append(name());
        return buf.toString();
    }

    @Override
    protected Field underlying() {
        return (Field) super.underlying();
    }
}
