/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.sun.jdi.AbsentInformationException;
import com.sun.jdi.Location;

/**
 *
 */
public class XyzzyxJDILocation extends XyzzyxJDIMirror implements Location {

    public XyzzyxJDILocation(XyzzyxJDIVirtualMachine xyzzyxvm, Location underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public long codeIndex() {
        return underlying().codeIndex();
    }

    @Override
    public XyzzyxJDIReferenceType declaringType() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().declaringType());
    }

    @Override
    public int lineNumber() {
        return underlying().lineNumber();
    }

    @Override
    public int lineNumber(String stratum) {
        return underlying().lineNumber(stratum);
    }

    @Override
    public XyzzyxJDIMethod method() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().method());
    }

    @Override
    public String sourceName() throws AbsentInformationException {
        return underlying().sourceName();
    }

    @Override
    public String sourceName(String stratum) throws AbsentInformationException {
        return underlying().sourceName(stratum);
    }

    @Override
    public String sourcePath() throws AbsentInformationException {
        return underlying().sourcePath();
    }

    @Override
    public String sourcePath(String stratum) throws AbsentInformationException {
        return underlying().sourcePath(stratum);
    }

    @Override
    public int compareTo(Location o) {
        return underlying().compareTo(XyzzyxJDIWrapper.unwrap(o));
    }

    @Override
    protected Location underlying() {
        return (Location) super.underlying();
    }
}