/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.sun.jdi.AbsentInformationException;
import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.LocalVariable;
import com.sun.jdi.Location;
import com.sun.jdi.Method;
import com.sun.jdi.Type;
import java.util.List;

/**
 *
 */
public class XyzzyxJDIMethod extends XyzzyxJDITypeComponent implements Method {
    public XyzzyxJDIMethod(XyzzyxJDIVirtualMachine xyzzyxvm, Method underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public List<Location> allLineLocations() throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocations(virtualMachine(), underlying().allLineLocations());
    }

    @Override
    public List<Location> allLineLocations(String stratum, String sourceName) throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocations(virtualMachine(), underlying().allLineLocations(stratum, sourceName));
    }

    @Override
    public List<String> argumentTypeNames() {
        return underlying().argumentTypeNames();
    }

    @Override
    public List<Type> argumentTypes() throws ClassNotLoadedException {
        return XyzzyxJDIWrapper.wrapTypes(virtualMachine(), underlying().argumentTypes());
    }

    @Override
    public List<LocalVariable> arguments() throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocalVariables(virtualMachine(), underlying().arguments());
    }

    @Override
    public byte[] bytecodes() {
        return underlying().bytecodes();
    }

    @Override
    public boolean isAbstract() {
        return underlying().isAbstract();
    }

    @Override
    public boolean isBridge() {
        return underlying().isBridge();
    }

    @Override
    public boolean isConstructor() {
        return underlying().isConstructor();
    }

    @Override
    public boolean isNative() {
        return underlying().isNative();
    }

    @Override
    public boolean isObsolete() {
        return underlying().isObsolete();
    }

    @Override
    public boolean isStaticInitializer() {
        return underlying().isStaticInitializer();
    }

    @Override
    public boolean isSynchronized() {
        return underlying().isSynchronized();
    }

    @Override
    public boolean isVarArgs() {
        return underlying().isVarArgs();
    }

    @Override
    public XyzzyxJDILocation location() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().location());
    }

    @Override
    public XyzzyxJDILocation locationOfCodeIndex(long index) {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().locationOfCodeIndex(index));
    }

    @Override
    public List<Location> locationsOfLine(int line) throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocations(virtualMachine(), underlying().locationsOfLine(line));
    }

    @Override
    public List<Location> locationsOfLine(String stratum, String sourceName, int line) throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocations(virtualMachine(), underlying().locationsOfLine(stratum, sourceName, line));
    }

    @Override
    public XyzzyxJDIType returnType() throws ClassNotLoadedException {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().returnType());
    }

    @Override
    public String returnTypeName() {
        return underlying().returnTypeName();
    }

    @Override
    public List<LocalVariable> variables() throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocalVariables(virtualMachine(), underlying().variables());
    }

    @Override
    public List<LocalVariable> variablesByName(String name) throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrapLocalVariables(virtualMachine(), underlying().variablesByName(name));
    }

    @Override
    public int compareTo(Method o) {
        return underlying().compareTo(XyzzyxJDIWrapper.unwrap(o));
    }

    @Override
    protected Method underlying() {
        return (Method) super.underlying();
    }
}
