/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.sun.jdi.AbsentInformationException;
import com.sun.jdi.ClassNotLoadedException;
import com.sun.jdi.InvalidTypeException;
import com.sun.jdi.LocalVariable;
import com.sun.jdi.StackFrame;
import com.sun.jdi.Value;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 */
public class XyzzyxJDIStackFrame extends XyzzyxJDIMirror implements StackFrame {
    public XyzzyxJDIStackFrame(XyzzyxJDIVirtualMachine xyzzyxvm, StackFrame underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public List<Value> getArgumentValues() {
        return XyzzyxJDIWrapper.wrapValues(virtualMachine(), underlying().getArgumentValues());
    }

    @Override
    public XyzzyxJDIValue getValue(LocalVariable var) {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().getValue(XyzzyxJDIWrapper.unwrap(var)));
    }

    @Override
    public Map<LocalVariable, Value> getValues(List<? extends LocalVariable> vars) {
        Map<LocalVariable, LocalVariable> fieldMap = new HashMap<>();
        List<LocalVariable> unwrappedLocalVariables = new ArrayList<>();
        for (LocalVariable var : vars) {
            LocalVariable unwrapped = XyzzyxJDIWrapper.unwrap(var);
            unwrappedLocalVariables.add(unwrapped);
            fieldMap.put(unwrapped, var);
        }
        Map<LocalVariable, Value> fieldValues = underlying().getValues(unwrappedLocalVariables);
        Map<LocalVariable, Value> result = new HashMap<>();
        for (Map.Entry<LocalVariable, Value> entry: fieldValues.entrySet()) {
            result.put(fieldMap.get(entry.getKey()), entry.getValue());
        }
        return result;
    }

    @Override
    public XyzzyxJDILocation location() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().location());
    }

    @Override
    public void setValue(LocalVariable var, Value value) throws InvalidTypeException, ClassNotLoadedException {
        underlying().setValue(XyzzyxJDIWrapper.unwrap(var), XyzzyxJDIWrapper.unwrap(value));
    }

    @Override
    public XyzzyxJDIObjectReference thisObject() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().thisObject());
    }

    @Override
    public XyzzyxJDIThreadReference thread() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().thread());
    }

    @Override
    public XyzzyxJDILocalVariable visibleVariableByName(String name) throws AbsentInformationException {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().visibleVariableByName(name));
    }

    @Override
    public List<LocalVariable> visibleVariables() throws AbsentInformationException {
        List<LocalVariable> locals = underlying().visibleVariables();
        List<LocalVariable> result = new ArrayList<>();
        for (LocalVariable var : locals) {
            result.add(XyzzyxJDIWrapper.wrap(virtualMachine(), var));
        }
        return result;
    }

    @Override
    protected StackFrame underlying() {
        return (StackFrame) super.underlying();
    }
}
