/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi;

import com.oracle.xyzzyx.jdi.event.XyzzyxJDIEventQueue;
import com.oracle.xyzzyx.jdi.request.XyzzyxJDIEventRequestManager;

import com.sun.jdi.Type;
import com.sun.jdi.ArrayReference;
import com.sun.jdi.ArrayType;
import com.sun.jdi.BooleanType;
import com.sun.jdi.BooleanValue;
import com.sun.jdi.ByteType;
import com.sun.jdi.ByteValue;
import com.sun.jdi.CharType;
import com.sun.jdi.CharValue;
import com.sun.jdi.ClassLoaderReference;
import com.sun.jdi.ClassObjectReference;
import com.sun.jdi.ClassType;
import com.sun.jdi.DoubleType;
import com.sun.jdi.DoubleValue;
import com.sun.jdi.Field;
import com.sun.jdi.FloatType;
import com.sun.jdi.FloatValue;
import com.sun.jdi.IntegerType;
import com.sun.jdi.IntegerValue;
import com.sun.jdi.InterfaceType;
import com.sun.jdi.LocalVariable;
import com.sun.jdi.Location;
import com.sun.jdi.LongType;
import com.sun.jdi.LongValue;
import com.sun.jdi.Method;
import com.sun.jdi.MonitorInfo;
import com.sun.jdi.ObjectReference;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.ShortType;
import com.sun.jdi.ShortValue;
import com.sun.jdi.StackFrame;
import com.sun.jdi.StringReference;
import com.sun.jdi.ThreadGroupReference;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.VirtualMachine;
import com.sun.jdi.Value;
import com.sun.jdi.VoidType;
import com.sun.jdi.VoidValue;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 */
public class XyzzyxJDIVirtualMachine extends XyzzyxJDIMirror implements VirtualMachine {
    private final boolean xyzzyxObjects;
    private final boolean xyzzyxCodeOnly;
    private final boolean xyzzyxSkiptoScript;

    public XyzzyxJDIVirtualMachine(VirtualMachine underlying) {
        super(null, underlying);

        this.xyzzyxObjects      = true;
        this.xyzzyxCodeOnly     = true;
        this.xyzzyxSkiptoScript = true;
    }

    public XyzzyxJDIVirtualMachine(VirtualMachine underlying,
                                    boolean xyzzyxObjects,
                                    boolean xyzzyxCodeOnly,
                                    boolean xyzzyxSkiptoScript) {
        super(null, underlying);

        this.xyzzyxObjects      = xyzzyxObjects;
        this.xyzzyxCodeOnly     = xyzzyxCodeOnly;
        this.xyzzyxSkiptoScript = xyzzyxSkiptoScript;
    }

    @Override
    public boolean canForceEarlyReturn() {
        return underlying().canForceEarlyReturn();
    }

    @Override
    public boolean canGetClassFileVersion() {
        return underlying().canGetClassFileVersion();
    }

    @Override
    public boolean canGetConstantPool() {
        return underlying().canGetConstantPool();
    }

    @Override
    public boolean canGetInstanceInfo() {
        return underlying().canGetInstanceInfo();
    }

    @Override
    public boolean canGetMethodReturnValues() {
        return underlying().canGetMethodReturnValues();
    }

    @Override
    public boolean canGetMonitorFrameInfo() {
        return underlying().canGetMonitorFrameInfo();
    }

    @Override
    public boolean canRequestMonitorEvents() {
        return underlying().canRequestMonitorEvents();
    }

    @Override
    public boolean canUseSourceNameFilters() {
        return underlying().canUseSourceNameFilters();
    }

    @Override
    public long[] instanceCounts(List<? extends ReferenceType> refTypes) {
        return underlying().instanceCounts(XyzzyxJDIWrapper.unwrapReferenceTypes(refTypes));
    }

    @Override
    public VoidValue mirrorOfVoid() {
        return voidValue();
    }

    @Override
    public XyzzyxJDIVirtualMachine virtualMachine() {
        return this;
    }

    @Override
    public List<ReferenceType> allClasses() {
        return XyzzyxJDIWrapper.wrapReferenceTypes(this, underlying().allClasses());
    }

    @Override
    public List<ThreadReference> allThreads() {
        return XyzzyxJDIWrapper.wrapThreads(this, underlying().allThreads());
    }

    @Override
    public boolean canAddMethod() {
        return underlying().canAddMethod();
    }

    @Override
    public boolean canBeModified() {
        return underlying().canBeModified();
    }

    @Override
    public boolean canGetBytecodes() {
        return underlying().canGetBytecodes();
    }

    @Override
    public boolean canGetCurrentContendedMonitor() {
        return underlying().canGetCurrentContendedMonitor();
    }

    @Override
    public boolean canGetMonitorInfo() {
        return underlying().canGetMonitorInfo();
    }

    @Override
    public boolean canGetOwnedMonitorInfo() {
        return underlying().canGetOwnedMonitorInfo();
    }

    @Override
    public boolean canGetSourceDebugExtension() {
        return underlying().canGetSourceDebugExtension();
    }

    @Override
    public boolean canGetSyntheticAttribute() {
        return underlying().canGetSyntheticAttribute();
    }

    @Override
    public boolean canPopFrames() {
        return underlying().canPopFrames();
    }

    @Override
    public boolean canRedefineClasses() {
        return underlying().canRedefineClasses();
    }

    @Override
    public boolean canRequestVMDeathEvent() {
        return underlying().canRequestVMDeathEvent();
    }

    @Override
    public boolean canUnrestrictedlyRedefineClasses() {
        return underlying().canUnrestrictedlyRedefineClasses();
    }

    @Override
    public boolean canUseInstanceFilters() {
        return underlying().canUseInstanceFilters();
    }

    @Override
    public boolean canWatchFieldAccess() {
        return underlying().canWatchFieldAccess();
    }

    @Override
    public boolean canWatchFieldModification() {
        return underlying().canWatchFieldModification();
    }

    @Override
    public List<ReferenceType> classesByName(String name) {
        List<ReferenceType> refTypes = underlying().classesByName(name);
        return XyzzyxJDIWrapper.wrapReferenceTypes(this, refTypes);
    }

    @Override
    public String description() {
        return underlying().description();
    }

    @Override
    public void dispose() {
        underlying().dispose();
    }


    private XyzzyxJDIEventQueue evtQueue;
    @Override
    public synchronized XyzzyxJDIEventQueue eventQueue() {
        if (evtQueue == null) {
            evtQueue = XyzzyxJDIWrapper.wrap(this, underlying().eventQueue());
        }
        return evtQueue;
    }

    private XyzzyxJDIEventRequestManager evtManager;
    @Override
    public synchronized XyzzyxJDIEventRequestManager eventRequestManager() {
        if (evtManager == null) {
            evtManager = XyzzyxJDIWrapper.wrap(this, underlying().eventRequestManager());
        }
        return evtManager;
    }

    @Override
    public void exit(int exitCode) {
        underlying().exit(exitCode);
    }

    @Override
    public String getDefaultStratum() {
        return underlying().getDefaultStratum();
    }

    @Override
    public XyzzyxJDIBooleanValue mirrorOf(boolean value) {
        return new XyzzyxJDIBooleanValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDIByteValue mirrorOf(byte value) {
        return new XyzzyxJDIByteValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDICharValue mirrorOf(char value) {
        return new XyzzyxJDICharValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDIShortValue mirrorOf(short value) {
        return new XyzzyxJDIShortValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDIIntegerValue mirrorOf(int value) {
        return new XyzzyxJDIIntegerValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDILongValue mirrorOf(long value) {
        return new XyzzyxJDILongValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDIFloatValue mirrorOf(float value) {
        return new XyzzyxJDIFloatValue(this, underlying().mirrorOf(value));
    }

    @Override
    public XyzzyxJDIDoubleValue mirrorOf(double value) {
        return new XyzzyxJDIDoubleValue(this, underlying().mirrorOf(value));
    }

    @Override
    public StringReference mirrorOf(String value) {
        return new XyzzyxJDIStringReference(this, underlying().mirrorOf(value));
    }

    // default values

    private XyzzyxJDIBooleanValue booleanDefaultValue;
    protected synchronized XyzzyxJDIBooleanValue booleanDefaultValue() {
        if (booleanDefaultValue == null) {
            booleanDefaultValue = mirrorOf(false);
        }
        return booleanDefaultValue;
    }

    private XyzzyxJDIByteValue byteDefaultValue;
    protected synchronized XyzzyxJDIByteValue byteDefaultValue() {
        if (byteDefaultValue == null) {
            byteDefaultValue = mirrorOf((byte)0);
        }
        return byteDefaultValue;
    }

    private XyzzyxJDICharValue charDefaultValue;
    protected synchronized XyzzyxJDICharValue charDefaultValue() {
        if (charDefaultValue == null) {
            charDefaultValue = mirrorOf('\u0000');
        }
        return charDefaultValue;
    }

    private XyzzyxJDIShortValue shortDefaultValue;
    protected synchronized XyzzyxJDIShortValue shortDefaultValue() {
        if (shortDefaultValue == null) {
            shortDefaultValue = mirrorOf((short)0);
        }
        return shortDefaultValue;
    }

    private XyzzyxJDIIntegerValue integerDefaultValue;
    protected synchronized XyzzyxJDIIntegerValue integerDefaultValue() {
        if (integerDefaultValue == null) {
            integerDefaultValue = mirrorOf(0);
        }
        return integerDefaultValue;
    }

    private XyzzyxJDILongValue longDefaultValue;
    protected synchronized XyzzyxJDILongValue longDefaultValue() {
        if (longDefaultValue == null) {
            longDefaultValue = mirrorOf(0l);
        }
        return longDefaultValue;
    }

    private XyzzyxJDIFloatValue floatDefaultValue;
    protected synchronized XyzzyxJDIFloatValue floatDefaultValue() {
        if (floatDefaultValue == null) {
            floatDefaultValue = mirrorOf(0.0f);
        }
        return floatDefaultValue;
    }

    private XyzzyxJDIDoubleValue doubleDefaultValue;
    protected synchronized XyzzyxJDIDoubleValue doubleDefaultValue() {
        if (doubleDefaultValue == null) {
            doubleDefaultValue = mirrorOf(0.0d);
        }
        return doubleDefaultValue;
    }

    //////////

    @Override
    public String name() {
        return underlying().name();
    }

    @Override
    public Process process() {
        return underlying().process();
    }

    @Override
    public void redefineClasses(Map<? extends ReferenceType, byte[]> classBytes) {
        Map<ReferenceType, byte[]> unwrappedClassBytes = new HashMap<>();
        for (Map.Entry<? extends ReferenceType, byte[]> entry : classBytes.entrySet()) {
            unwrappedClassBytes.put(XyzzyxJDIWrapper.unwrap(entry.getKey()), entry.getValue());
        }
        underlying().redefineClasses(unwrappedClassBytes);
    }

    @Override
    public void resume() {
        underlying().resume();
    }

    @Override
    public void setDebugTraceMode(int mode) {
        underlying().setDebugTraceMode(mode);
    }

    @Override
    public void setDefaultStratum(String stratum) {
        underlying().setDefaultStratum(stratum);
    }

    @Override
    public void suspend() {
        underlying().suspend();
    }

    @Override
    public List<ThreadGroupReference> topLevelThreadGroups() {
        return XyzzyxJDIWrapper.wrapThreadGroups(this, underlying().topLevelThreadGroups());
    }

    @Override
    public String version() {
        return underlying().version();
    }

    private XyzzyxJDIThreadReference cacheRunnngThread = null;

    public XyzzyxJDIThreadReference runningThread() {
        if (cacheRunnngThread == null) {
            List<ThreadReference> allThreads = allThreads();
            cacheRunnngThread = XyzzyxJDIWrapper.wrap(this, allThreads.get(allThreads.size() - 1));
            for (ThreadReference thread : allThreads) {
                if ("main".equals(thread.name())) {
                    cacheRunnngThread = XyzzyxJDIWrapper.wrap(this, thread);
                    break;
                }
            }
        }
        return cacheRunnngThread;
    }

    @Override
    protected VirtualMachine underlying() {
        return (VirtualMachine) super.underlying();
    }

    public static final String NASHORNJDI_OBJECT_TYPE_NAME = "com.oracle.xyzzyx.runtime.ScriptObject";
    private XyzzyxJDIObjectType xyzzyxObjectType;
    public synchronized XyzzyxJDIObjectType xyzzyxObjectType() {
        if (xyzzyxObjectType == null) {
            List<ReferenceType> refTypes = classesByName(NASHORNJDI_OBJECT_TYPE_NAME);
            xyzzyxObjectType = refTypes.isEmpty() ? null : (XyzzyxJDIObjectType) refTypes.get(0);
        }
        return xyzzyxObjectType;
    }

    public boolean subclassOf(ClassType subClass, ClassType superClass) {
        while (subClass != null) {
            if (superClass == subClass) {
                return true;
            }
            subClass = subClass.superclass();
        }
        return false;
    }

    private XyzzyxJDIVoidValue voidValue;
    protected synchronized XyzzyxJDIVoidValue voidValue() {
        if (voidValue == null) {
            voidValue = new XyzzyxJDIVoidValue(this, underlying().mirrorOfVoid());
        }
        return voidValue;
    }

    // wrapper methods

    // primitive type accessors
    private XyzzyxJDIVoidType voidType;
    protected synchronized XyzzyxJDIVoidType voidType(VoidType vt) {
        if (voidType == null) {
            voidType = new XyzzyxJDIVoidType(this, vt);
        }
        return voidType;
    }

    private XyzzyxJDIBooleanType booleanType;
    protected synchronized XyzzyxJDIBooleanType booleanType(BooleanType bt) {
        if (booleanType == null) {
            booleanType = new XyzzyxJDIBooleanType(this, bt);
        }
        return booleanType;
    }

    private XyzzyxJDICharType charType;
    protected synchronized XyzzyxJDICharType charType(CharType ct) {
        if (charType == null) {
            charType = new XyzzyxJDICharType(this, ct);
        }
        return charType;
    }

    private XyzzyxJDIByteType byteType;
    protected synchronized XyzzyxJDIByteType byteType(ByteType bt) {
        if (byteType == null) {
            byteType = new XyzzyxJDIByteType(this, bt);
        }
        return byteType;
    }

    private XyzzyxJDIShortType shortType;
    protected synchronized XyzzyxJDIShortType shortType(ShortType st) {
        if (shortType == null) {
            shortType = new XyzzyxJDIShortType(this, st);
        }
        return shortType;
    }

    private XyzzyxJDIIntegerType integerType;
    protected synchronized XyzzyxJDIIntegerType integerType(IntegerType it) {
        if (integerType == null) {
            integerType = new XyzzyxJDIIntegerType(this, it);
        }
        return integerType;
    }

    private XyzzyxJDILongType longType;
    protected synchronized XyzzyxJDILongType longType(LongType lt) {
        if (longType == null) {
            longType = new XyzzyxJDILongType(this, lt);
        }
        return longType;
    }

    private XyzzyxJDIFloatType floatType;
    protected synchronized XyzzyxJDIFloatType floatType(FloatType ft) {
        if (floatType == null) {
            floatType = new XyzzyxJDIFloatType(this, ft);
        }
        return floatType;
    }


    private XyzzyxJDIDoubleType doubleType;
    protected synchronized XyzzyxJDIDoubleType doubleType(DoubleType dt) {
        if (doubleType == null) {
            doubleType = new XyzzyxJDIDoubleType(this, dt);
        }
        return doubleType;
    }

    protected XyzzyxJDILocation location(Location loc) {
        return new XyzzyxJDILocation(this, loc);
    }

    private final Map<ReferenceType, XyzzyxJDIReferenceType> refTypesCache =
            new HashMap<>();

    protected XyzzyxJDIReferenceType referenceType(ReferenceType rt) {
        synchronized (refTypesCache) {
            if (! refTypesCache.containsKey(rt)) {
                refTypesCache.put(rt, new XyzzyxJDIReferenceType(this, rt));
            }
            return refTypesCache.get(rt);
        }
    }

    protected XyzzyxJDIClassType classType(ClassType ct) {
        synchronized (refTypesCache) {
            if (! refTypesCache.containsKey(ct)) {
                if (ct.name().equals(NASHORNJDI_OBJECT_TYPE_NAME)) {
                    refTypesCache.put(ct, new XyzzyxJDIObjectType(this, ct));
                } else {
                    refTypesCache.put(ct, new XyzzyxJDIClassType(this, ct));
                }
            }
            return (XyzzyxJDIClassType) refTypesCache.get(ct);
        }
    }

    protected XyzzyxJDIInterfaceType interfaceType(InterfaceType it) {
        synchronized (refTypesCache) {
            if (! refTypesCache.containsKey(it)) {
                refTypesCache.put(it, new XyzzyxJDIInterfaceType(this, it));
            }
            return (XyzzyxJDIInterfaceType) refTypesCache.get(it);
        }
    }

    protected XyzzyxJDIArrayType arrayType(ArrayType at) {
        synchronized (at) {
            if (! refTypesCache.containsKey(at)) {
                refTypesCache.put(at, new XyzzyxJDIArrayType(this, at));
            }
            return (XyzzyxJDIArrayType) refTypesCache.get(at);
        }
    }

    protected XyzzyxJDIField field(Field field) {
        return new XyzzyxJDIField(this, field);
    }

    protected XyzzyxJDIMethod method(Method method) {
        return new XyzzyxJDIMethod(this, method);
    }

    protected XyzzyxJDILocalVariable localVariable(LocalVariable var) {
        return new XyzzyxJDILocalVariable(this, var);
    }

    protected XyzzyxJDIBooleanValue booleanValue(BooleanValue value) {
        return new XyzzyxJDIBooleanValue(this, value);
    }

    protected XyzzyxJDICharValue charValue(CharValue value) {
        return new XyzzyxJDICharValue(this, value);
    }

    protected XyzzyxJDIByteValue byteValue(ByteValue value) {
        return new XyzzyxJDIByteValue(this, value);
    }

    protected XyzzyxJDIShortValue shortValue(ShortValue value) {
        return new XyzzyxJDIShortValue(this, value);
    }

    protected XyzzyxJDIIntegerValue integerValue(IntegerValue value) {
        return new XyzzyxJDIIntegerValue(this, value);
    }

    protected XyzzyxJDILongValue longValue(LongValue value) {
        return new XyzzyxJDILongValue(this, value);
    }

    protected XyzzyxJDIFloatValue floatValue(FloatValue value) {
        return new XyzzyxJDIFloatValue(this, value);
    }

    protected XyzzyxJDIDoubleValue doubleValue(DoubleValue value) {
        return new XyzzyxJDIDoubleValue(this, value);
    }

    protected XyzzyxJDIObjectReference objectReference(ObjectReference ref) {
        return new XyzzyxJDIObjectReference(this, ref);
    }

    protected XyzzyxJDIThreadReference threadReference(ThreadReference tref) {
        return new XyzzyxJDIThreadReference(this, tref);
    }

    protected XyzzyxJDIThreadGroupReference threadGroupReference(ThreadGroupReference tgref) {
        return new XyzzyxJDIThreadGroupReference(this, tgref);
    }

    protected XyzzyxJDIStringReference stringReference(StringReference sref) {
        return new XyzzyxJDIStringReference(this, sref);
    }

    protected XyzzyxJDIClassLoaderReference classLoaderReference(ClassLoaderReference clref) {
        return new XyzzyxJDIClassLoaderReference(this, clref);
    }

    protected XyzzyxJDIClassObjectReference classObjectReference(ClassObjectReference coref) {
        return new XyzzyxJDIClassObjectReference(this, coref);
    }

    protected XyzzyxJDIArrayReference arrayReference(ArrayReference aref) {
        return new XyzzyxJDIArrayReference(this, aref);
    }

    protected XyzzyxJDIMonitorInfo monitorInfo(MonitorInfo monitorInfo) {
        return new XyzzyxJDIMonitorInfo(this, monitorInfo);
    }

    protected XyzzyxJDIStackFrame stackFrame(StackFrame frame) {
        return new XyzzyxJDIStackFrame(this, frame);
    }

    protected Exception lastFieldAccessException = null;
    protected void setLastFieldAccessException(Exception ee) {
        lastFieldAccessException = ee;
    }

    /**
     * JDI addition: Return the exception thrown by an invokeMethod call that was
     * performed in the most recent setValue, getValue, or getValues method call.
     * Return null if no such exception was thrown.
     */
    public Exception lastFieldAccessException() {
        return lastFieldAccessException;
    }

    protected Value defaultValue(Type type) {
        if (type instanceof BooleanType) {
            return booleanDefaultValue();
        }
        if (type instanceof ByteType) {
            return byteDefaultValue();
        }
        if (type instanceof CharType) {
            return charDefaultValue();
        }
        if (type instanceof DoubleType) {
            return doubleDefaultValue();
        }
        if (type instanceof FloatType) {
            return floatDefaultValue();
        }
        if (type instanceof IntegerType) {
            return integerDefaultValue();
        }
        if (type instanceof LongType) {
            return longDefaultValue();
        }
        if (type instanceof ShortType) {
            return shortDefaultValue();
        }
        // else it is an object/array...
        return null;
    }
}
