/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi.connect;

import com.oracle.xyzzyx.jdi.XyzzyxJDIVirtualMachine;
import com.oracle.xyzzyx.jdi.XyzzyxJDIWrapper;

import com.sun.jdi.connect.AttachingConnector;
import com.sun.jdi.connect.Connector.Argument;
import com.sun.jdi.connect.IllegalConnectorArgumentsException;

import java.io.IOException;
import java.util.Map;

/**
 *
 */
public class XyzzyxJDISocketAttachingConnector extends XyzzyxJDIConnector implements AttachingConnector {
    public XyzzyxJDISocketAttachingConnector() {
        this(makePlatformConnector());
    }

    public XyzzyxJDISocketAttachingConnector(AttachingConnector underlying) {
        super(underlying);
    }

    @Override
    public XyzzyxJDIVirtualMachine attach(Map<String, ? extends Argument> args)
            throws IOException, IllegalConnectorArgumentsException {
        return XyzzyxJDIWrapper.wrap(underlying().attach(args));
    }

    @Override
    protected AttachingConnector underlying() {
        return (AttachingConnector) super.underlying();
    }

    private static final String SOCKET_ATTACHING_CONN = "com.sun.tools.jdi.SocketAttachingConnector";
    private static AttachingConnector makePlatformConnector() {
        Class connectorClass = null;
        try {
            connectorClass = Class.forName(SOCKET_ATTACHING_CONN);
        } catch (ClassNotFoundException cnfe) {}
        if (connectorClass == null) {
            throw new RuntimeException("can not load class: " + SOCKET_ATTACHING_CONN);
        }
        try {
            return (AttachingConnector) connectorClass.newInstance();
        } catch (InstantiationException | IllegalAccessException exp) {
            throw new RuntimeException(exp);
        }
    }
}