/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi.event;

import com.oracle.xyzzyx.jdi.XyzzyxJDIVirtualMachine;

import com.sun.jdi.event.EventIterator;

/**
 *
 */
public class XyzzyxJDIEventIterator implements EventIterator {
    private final XyzzyxJDIVirtualMachine xyzzyxvm;
    private final EventIterator underlying;

    public XyzzyxJDIEventIterator(XyzzyxJDIVirtualMachine xyzzyxvm, EventIterator underlying) {
        this.xyzzyxvm = xyzzyxvm;
        this.underlying = underlying;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof XyzzyxJDIEventIterator) {
            obj = ((XyzzyxJDIEventIterator)obj).underlying();
        }
        return underlying().equals(obj);
    }

    @Override
    public int hashCode() {
        return underlying().hashCode();
    }

    @Override
    public String toString() {
        return underlying().toString();
    }

    public XyzzyxJDIEvent nextEvent() {
        return XyzzyxJDIEvent.wrap(virtualMachine(), underlying().nextEvent());
    }

    public boolean hasNext() {
        return underlying().hasNext();
    }

    public XyzzyxJDIEvent next() {
        return XyzzyxJDIEvent.wrap(virtualMachine(), underlying().next());
    }

    public void remove() {
        underlying().remove();
    }

    protected XyzzyxJDIVirtualMachine virtualMachine() {
        return xyzzyxvm;
    }

    protected EventIterator underlying() {
        return underlying;
    }

    public static XyzzyxJDIEventIterator wrap(XyzzyxJDIVirtualMachine xyzzyxvm, EventIterator evtItr) {
        return (evtItr == null)? null : new XyzzyxJDIEventIterator(xyzzyxvm, evtItr);
    }
}