/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi.event;

import com.oracle.xyzzyx.jdi.XyzzyxJDIMirror;
import com.oracle.xyzzyx.jdi.XyzzyxJDIVirtualMachine;

import com.sun.jdi.event.Event;
import com.sun.jdi.event.EventSet;

import java.util.Collection;
import java.util.Iterator;

/**
 *
 */
public class XyzzyxJDIEventSet extends XyzzyxJDIMirror implements EventSet {
    public XyzzyxJDIEventSet(XyzzyxJDIVirtualMachine xyzzyxvm, EventSet underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public XyzzyxJDIEventIterator eventIterator() {
        return XyzzyxJDIEventIterator.wrap(virtualMachine(), underlying().eventIterator());
    }

    @Override
    public void resume() {
        underlying().resume();
    }

    @Override
    public int suspendPolicy() {
        return underlying().suspendPolicy();
    }

    @Override
    public boolean add(Event evt) {
        return underlying().add(XyzzyxJDIEvent.unwrap(evt));
    }

    @Override
    public boolean addAll(Collection<? extends Event> events) {
        return underlying().addAll(XyzzyxJDIEvent.unwrapEvents(events));
    }

    @Override
    public void clear() {
        underlying().clear();
    }

    @Override
    public boolean contains(Object obj) {
        return underlying().contains((obj instanceof Event)? XyzzyxJDIEvent.unwrap((Event)obj) : obj);
    }

    @Override
    public boolean containsAll(Collection<?> arg0) {
        return underlying().containsAll(XyzzyxJDIEvent.unwrapEvents(arg0));
    }

    @Override
    public boolean isEmpty() {
        return underlying().isEmpty();
    }

    @Override
    public Iterator<Event> iterator() {
        final Iterator<Event> wrapped = underlying().iterator();
        if (wrapped != null) {
            return new Iterator<Event>() {
                @Override
                public boolean hasNext() {
                    return wrapped.hasNext();
                }

                @Override
                public Event next() {
                    return XyzzyxJDIEvent.wrap(virtualMachine(), wrapped.next());
                }

                @Override
                public void remove() {
                    wrapped.remove();
                }
            };
        } else {
            return null;
        }
    }

    @Override
    public boolean remove(Object obj) {
        return underlying().remove((obj instanceof Event)?
            XyzzyxJDIEvent.unwrap((Event)obj) : obj);
    }

    @Override
    public boolean removeAll(Collection<?> arg0) {
        return underlying().removeAll(XyzzyxJDIEvent.unwrapEvents(arg0));
    }

    @Override
    public boolean retainAll(Collection<?> arg0) {
        return underlying().retainAll(XyzzyxJDIEvent.unwrapEvents(arg0));
    }

    @Override
    public int size() {
        return underlying().size();
    }

    @Override
    public Object[] toArray() {
        Object[] res = underlying().toArray();
        if (res != null) {
            for (int i = 0; i < res.length; i++) {
                if (res[i] instanceof Event) {
                    res[i] = XyzzyxJDIEvent.unwrap((Event)res[i]);
                }
            }
        }
        return res;
    }

    @Override
    public <T> T[] toArray(T[] arg0) {
        // FIXME
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    protected EventSet underlying() {
        return (EventSet) super.underlying();
    }

    public static XyzzyxJDIEventSet wrap(XyzzyxJDIVirtualMachine xyzzyxvm, EventSet evtSet) {
        return (evtSet == null)? null : new XyzzyxJDIEventSet(xyzzyxvm, evtSet);
    }
}