/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi.request;

import com.oracle.xyzzyx.jdi.XyzzyxJDIMirror;
import com.oracle.xyzzyx.jdi.XyzzyxJDIVirtualMachine;
import com.oracle.xyzzyx.jdi.XyzzyxJDIWrapper;

import com.sun.jdi.Field;
import com.sun.jdi.Location;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.request.AccessWatchpointRequest;
import com.sun.jdi.request.BreakpointRequest;
import com.sun.jdi.request.ClassPrepareRequest;
import com.sun.jdi.request.ClassUnloadRequest;
import com.sun.jdi.request.EventRequest;
import com.sun.jdi.request.EventRequestManager;
import com.sun.jdi.request.ExceptionRequest;
import com.sun.jdi.request.MethodEntryRequest;
import com.sun.jdi.request.MethodExitRequest;
import com.sun.jdi.request.ModificationWatchpointRequest;
import com.sun.jdi.request.MonitorContendedEnterRequest;
import com.sun.jdi.request.MonitorContendedEnteredRequest;
import com.sun.jdi.request.MonitorWaitRequest;
import com.sun.jdi.request.MonitorWaitedRequest;
import com.sun.jdi.request.StepRequest;
import com.sun.jdi.request.ThreadDeathRequest;
import com.sun.jdi.request.ThreadStartRequest;
import com.sun.jdi.request.VMDeathRequest;
import java.util.List;

/**
 *
 */
public class XyzzyxJDIEventRequestManager extends XyzzyxJDIMirror implements EventRequestManager {
    public XyzzyxJDIEventRequestManager(XyzzyxJDIVirtualMachine xyzzyxvm, EventRequestManager underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public List<AccessWatchpointRequest> accessWatchpointRequests() {
        return XyzzyxJDIEventRequest.wrapAccessWatchpointRequests(virtualMachine(),
                underlying().accessWatchpointRequests());
    }

    @Override
    public List<BreakpointRequest> breakpointRequests() {
        return XyzzyxJDIEventRequest.wrapBreakpointRequests(virtualMachine(),
                underlying().breakpointRequests());
    }

    @Override
    public List<ClassPrepareRequest> classPrepareRequests() {
        return XyzzyxJDIEventRequest.wrapClassPrepareRequests(virtualMachine(),
                underlying().classPrepareRequests());
    }

    @Override
    public List<ClassUnloadRequest> classUnloadRequests() {
        return XyzzyxJDIEventRequest.wrapClassUnloadRequests(virtualMachine(),
                underlying().classUnloadRequests());
    }

    @Override
    public XyzzyxJDIAccessWatchpointRequest createAccessWatchpointRequest(Field arg0) {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createAccessWatchpointRequest(XyzzyxJDIWrapper.unwrap(arg0)));
    }

    @Override
    public XyzzyxJDIBreakpointRequest createBreakpointRequest(Location arg0) {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createBreakpointRequest(XyzzyxJDIWrapper.unwrap(arg0)));
    }

    @Override
    public XyzzyxJDIClassPrepareRequest createClassPrepareRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createClassPrepareRequest());
    }

    @Override
    public XyzzyxJDIClassUnloadRequest createClassUnloadRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createClassUnloadRequest());
    }

    @Override
    public XyzzyxJDIExceptionRequest createExceptionRequest(ReferenceType arg0, boolean arg1, boolean arg2) {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createExceptionRequest(XyzzyxJDIWrapper.unwrap(arg0), arg1, arg2));
    }

    @Override
    public XyzzyxJDIMethodEntryRequest createMethodEntryRequest() {
        XyzzyxJDIMethodEntryRequest request = XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createMethodEntryRequest());
        // Add filters and exclusions to request here.
        return request;
    }

    @Override
    public XyzzyxJDIMethodExitRequest createMethodExitRequest() {
        XyzzyxJDIMethodExitRequest request = XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createMethodExitRequest());
        // Add filters and exclusions to request here.
        return request;
    }

    @Override
    public XyzzyxJDIModificationWatchpointRequest createModificationWatchpointRequest(Field arg0) {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createModificationWatchpointRequest(XyzzyxJDIWrapper.unwrap(arg0)));
    }

    @Override
    public XyzzyxJDIMonitorContendedEnterRequest createMonitorContendedEnterRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createMonitorContendedEnterRequest());
    }

    @Override
    public XyzzyxJDIMonitorContendedEnteredRequest createMonitorContendedEnteredRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createMonitorContendedEnteredRequest());
    }

    @Override
    public XyzzyxJDIMonitorWaitRequest createMonitorWaitRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createMonitorWaitRequest());
    }

    @Override
    public XyzzyxJDIMonitorWaitedRequest createMonitorWaitedRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createMonitorWaitedRequest());
    }

    @Override
    public XyzzyxJDIStepRequest createStepRequest(ThreadReference arg0, int arg1, int arg2) {
        StepRequest request = underlying().createStepRequest(XyzzyxJDIWrapper.unwrap(arg0), arg1, arg2);
        // Add filters and exclusions to request here.
        return XyzzyxJDIEventRequest.wrap(virtualMachine(), request);
    }

    @Override
    public XyzzyxJDIThreadDeathRequest createThreadDeathRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createThreadDeathRequest());
    }

    @Override
    public XyzzyxJDIThreadStartRequest createThreadStartRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createThreadStartRequest());
    }

    @Override
    public XyzzyxJDIVMDeathRequest createVMDeathRequest() {
        return XyzzyxJDIEventRequest.wrap(virtualMachine(),
                underlying().createVMDeathRequest());
    }

    @Override
    public void deleteAllBreakpoints() {
        underlying().deleteAllBreakpoints();
    }

    @Override
    public void deleteEventRequest(EventRequest arg0) {
        underlying().deleteEventRequest(XyzzyxJDIEventRequest.unwrap(arg0));
    }

    @Override
    public void deleteEventRequests(List<? extends EventRequest> arg0) {
        underlying().deleteEventRequests(XyzzyxJDIEventRequest.unwrapEventRequests(arg0));
    }

    @Override
    public List<ExceptionRequest> exceptionRequests() {
        return XyzzyxJDIEventRequest.wrapExceptionRequests(virtualMachine(),
                underlying().exceptionRequests());
    }

    @Override
    public List<MethodEntryRequest> methodEntryRequests() {
        return XyzzyxJDIEventRequest.wrapMethodEntryRequests(virtualMachine(),
                underlying().methodEntryRequests());
    }

    @Override
    public List<MethodExitRequest> methodExitRequests() {
        return XyzzyxJDIEventRequest.wrapMethodExitRequests(virtualMachine(),
                underlying().methodExitRequests());
    }

    @Override
    public List<ModificationWatchpointRequest> modificationWatchpointRequests() {
        return XyzzyxJDIEventRequest.wrapModificationWatchpointRequests(virtualMachine(),
                underlying().modificationWatchpointRequests());
    }

    @Override
    public List<MonitorContendedEnterRequest> monitorContendedEnterRequests() {
        return XyzzyxJDIEventRequest.wrapMonitorContendedEnterRequests(virtualMachine(),
                underlying().monitorContendedEnterRequests());
    }

    @Override
    public List<MonitorContendedEnteredRequest> monitorContendedEnteredRequests() {
        return XyzzyxJDIEventRequest.wrapMonitorContendedEnteredRequests(virtualMachine(),
                underlying().monitorContendedEnteredRequests());
    }

    @Override
    public List<MonitorWaitRequest> monitorWaitRequests() {
        return XyzzyxJDIEventRequest.wrapMonitorWaitRequests(virtualMachine(),
                underlying().monitorWaitRequests());
    }

    @Override
    public List<MonitorWaitedRequest> monitorWaitedRequests() {
        return XyzzyxJDIEventRequest.wrapMonitorWaitedRequests(virtualMachine(),
                underlying().monitorWaitedRequests());
    }

    @Override
    public List<StepRequest> stepRequests() {
        return XyzzyxJDIEventRequest.wrapStepRequests(virtualMachine(),
                underlying().stepRequests());
    }

    @Override
    public List<ThreadDeathRequest> threadDeathRequests() {
        return XyzzyxJDIEventRequest.wrapThreadDeathRequests(virtualMachine(),
                underlying().threadDeathRequests());
    }

    @Override
    public List<ThreadStartRequest> threadStartRequests() {
        return XyzzyxJDIEventRequest.wrapThreadStartRequests(virtualMachine(),
                underlying().threadStartRequests());
    }

    @Override
    public List<VMDeathRequest> vmDeathRequests() {
        return XyzzyxJDIEventRequest.wrapVMDeathRequests(virtualMachine(),
                underlying().vmDeathRequests());
    }

    @Override
    protected EventRequestManager underlying() {
        return (EventRequestManager) super.underlying();
    }
}