/*
 * Copyright (c) 2010-2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.xyzzyx.jdi.request;

import com.oracle.xyzzyx.jdi.XyzzyxJDIVirtualMachine;
import com.oracle.xyzzyx.jdi.XyzzyxJDIWrapper;

import com.sun.jdi.Field;
import com.sun.jdi.ObjectReference;
import com.sun.jdi.ReferenceType;
import com.sun.jdi.ThreadReference;
import com.sun.jdi.request.ModificationWatchpointRequest;

/**
 *
 */
public class XyzzyxJDIModificationWatchpointRequest extends XyzzyxJDIEventRequest implements ModificationWatchpointRequest {
    public XyzzyxJDIModificationWatchpointRequest(XyzzyxJDIVirtualMachine xyzzyxvm, ModificationWatchpointRequest underlying) {
        super(xyzzyxvm, underlying);
    }

    @Override
    public void addClassExclusionFilter(String arg0) {
        underlying().addClassExclusionFilter(arg0);
    }

    @Override
    public void addClassFilter(ReferenceType arg0) {
        underlying().addClassFilter(XyzzyxJDIWrapper.unwrap(arg0));
    }

    @Override
    public void addClassFilter(String arg0) {
        underlying().addClassFilter(arg0);
    }

    @Override
    public void addInstanceFilter(ObjectReference arg0) {
        underlying().addInstanceFilter(XyzzyxJDIWrapper.unwrap(arg0));
    }

    @Override
    public void addThreadFilter(ThreadReference arg0) {
        underlying().addThreadFilter(XyzzyxJDIWrapper.unwrap(arg0));
    }

    @Override
    public Field field() {
        return XyzzyxJDIWrapper.wrap(virtualMachine(), underlying().field());
    }

    @Override
    protected ModificationWatchpointRequest underlying() {
        return (ModificationWatchpointRequest) super.underlying();
    }
}
